@@ -10,7 +10,7 @@ extern crate nom;
 extern crate chrono;
 
 use std::error::Error;
-use nom::{IResult, Err};
+use nom::{IResult};
 
 pub use self::parser::{Patch, File, FileMetadata};
 use self::parser::{patch};
@@ -43,23 +43,11 @@ impl Error for PatchError {
 
 pub fn parse(diff: &str) -> Result<Patch, PatchError> {
     match patch(diff.as_bytes()) {
-        IResult::Done(_, ((old, new), hunks, no_newline)) =>
-            Ok(Patch {
-                old: old,
-                new: new,
-                hunks: hunks,
-                no_newline: no_newline,
-            }),
-        IResult::Incomplete(x) => {
-            println!("incomplete {:?}", x);
-            Err(PatchError::ParseError)
-        },
-        IResult::Error(x) => {
-            if let Err::Position(_, chrs) = x {
-                println!("chrs {:?}", std::str::from_utf8(chrs));
-            }
-            // println!("{:?}", x);
-            Err(PatchError::ParseError)
-        },
+        IResult::Done(_, p) =>
+            Ok(p),
+        IResult::Incomplete(_) =>
+            Err(PatchError::ParseError),
+        IResult::Error(_) =>
+            Err(PatchError::ParseError),
     }
 }
